/*	Application		A program which allocates memory in 100,000 blocks each time the user presses a key.
	File			DynamicMemoryAllocation.Java
	Purpose			Demonstrates dynamic memory allocation
	Author			Richard John Anthony
	Date			March 2015

        To run the project from the command line, go to the dist folder and type: java -jar "DynamicMemoryAllocation.jar"	
*/

package DynamicMemoryAllocation;

import java.util.*;
import java.io.Console;

public class DynamicMemoryAllocation
{
    static int iHundredThousand = 100000;
    static Console con;

    public static void main(String[] args)
    {
        con = System.console();
        if (null == con)
        {
            System.out.println("!! Error accessing console: Run in a command window !!");
            System.exit(-1);
        }
        
        String sInput;
        LinkedList<byte[]> MemoryBlockList = new LinkedList<byte[]>(); // Declare a linked list of byte array objects
        int lTotalAllocation = 0;

        System.out.println("*******************************");
        System.out.println("*  Dynamic Memory Allocation  *");
        System.out.println("*******************************");

        while(true)
        {
            System.out.println("Type a key to allocate a 100,000 byte block of memory or type \"Quit\" to quit: ");
            try
            {
                sInput = con.readLine();
                if(sInput.length() >= 4 && sInput.substring(0, 4).toUpperCase().equals("QUIT"))
                {
                    System.out.println("Quitting\n");
                    System.exit(-1);
                }
            }
            catch (Exception Ex)
            {   // Control-C for example causes us to come here
                System.out.println("Quitting\n");
                System.exit(-1);
            }

            System.out.printf("Requesting %d bytes ... ", iHundredThousand);
            try
            {   // Allocate memory to a local variable
                byte[] bIncrementalBlockOfMemory = new byte[iHundredThousand];
                // Add the local variable into the linked list to retain a reference to it and thus prevent it being garbage-collected
                MemoryBlockList.add(bIncrementalBlockOfMemory);
                lTotalAllocation += iHundredThousand;
                System.out.printf("result:ALLOCATED  (Total now is: %d)\n", lTotalAllocation);
            }
            catch(OutOfMemoryError Ex) // There is insufficient memory to satisfy the request.
            {
                System.out.println("result: allocation REFUSED!\nQuitting\n");
                System.exit(-1);
            }
        }
    }
}       
        
        
